unit UFormMain;

{$mode delphi}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, TAGraph, TASeries, Forms, Controls, Graphics,
  Dialogs, StdCtrls, Menus, ComCtrls, ExtCtrls, ActnList, dateutils, syncobjs,
  UProject, UDriveScan, UPrefixMultiplier, ULastOpenedList, UPersistentForm,
  XMLConf, UPhysDrive;

type
  { TFormMain }

  TFormMain = class(TForm)
    AShowSettings: TAction;
    AShowAbout: TAction;
    AOperationAdd: TAction;
    AOperationRemove: TAction;
    AViewToolbar: TAction;
    ADriveSelect: TAction;
    AOperationOptions: TAction;
    AFileClose: TAction;
    AFileNew: TAction;
    AScanContinue: TAction;
    AFileSave: TAction;
    AFileOpen: TAction;
    AFileSaveAs: TAction;
    AExit: TAction;
    AProjectOptions: TAction;
    AScanStart: TAction;
    AScanStop: TAction;
    ActionList1: TActionList;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    ButtonScan: TButton;
    ButtonScan1: TButton;
    ButtonScan2: TButton;
    ChartSpeed: TChart;
    ChartSpeedLineSeriesAvg: TLineSeries;
    ChartSpeedLineSeriesMin: TLineSeries;
    ChartSpeedLineSeriesMax: TLineSeries;
    ComboBoxDrive: TComboBox;
    Image1: TImage;
    Label1: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    LabelBlockCurrent: TLabel;
    LabelBlockDamaged: TLabel;
    LabelElapsedTime: TLabel;
    LabelEstimatedTime: TLabel;
    LabelIOSpeed: TLabel;
    LabelSectorCount: TLabel;
    LabelSectorPerBlock: TLabel;
    LabelSectorSize: TLabel;
    LabelSize: TLabel;
    LastOpenedList1: TLastOpenedList;
    ListView1: TListView;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem10: TMenuItem;
    MenuItem11: TMenuItem;
    MenuItem12: TMenuItem;
    MenuItem13: TMenuItem;
    MenuItem14: TMenuItem;
    MenuItem15: TMenuItem;
    MenuItem16: TMenuItem;
    MenuItem17: TMenuItem;
    MenuItem18: TMenuItem;
    MenuItem19: TMenuItem;
    MenuItem20: TMenuItem;
    MenuItem21: TMenuItem;
    MenuItem22: TMenuItem;
    MenuItem23: TMenuItem;
    MenuItemOpenRecent: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    OpenDialog1: TOpenDialog;
    PageControl1: TPageControl;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    PopupMenu1: TPopupMenu;
    SaveDialog1: TSaveDialog;
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    TabSheetSectors: TTabSheet;
    TabSheetSpeed: TTabSheet;
    Timer1: TTimer;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    procedure AExitExecute(Sender: TObject);
    procedure AFileCloseExecute(Sender: TObject);
    procedure AFileNewExecute(Sender: TObject);
    procedure AFileOpenExecute(Sender: TObject);
    procedure AFileSaveAsExecute(Sender: TObject);
    procedure AFileSaveExecute(Sender: TObject);
    procedure AOperationAddExecute(Sender: TObject);
    procedure AOperationOptionsExecute(Sender: TObject);
    procedure AOperationRemoveExecute(Sender: TObject);
    procedure AProjectOptionsExecute(Sender: TObject);
    procedure AScanContinueExecute(Sender: TObject);
    procedure AScanStartExecute(Sender: TObject);
    procedure AScanStopExecute(Sender: TObject);
    procedure AShowAboutExecute(Sender: TObject);
    procedure AShowSettingsExecute(Sender: TObject);
    procedure AViewToolbarExecute(Sender: TObject);
    procedure ComboBoxDriveChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Image1Resize(Sender: TObject);
    procedure ListView1Data(Sender: TObject; Item: TListItem);
    procedure ListView1SelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure Timer1Timer(Sender: TObject);
  private
    PrefixMultiplier: TPrefixMultiplier;
    LastBlockPos: Integer;
    RedrawPending: Boolean;
    LastDriveName: string;
    ShowToolBar: Boolean;
    procedure OpenRecentClick(Sender: TObject);
    procedure ReloadOperationList;
    procedure ProjectOpen(FileName: string);
    procedure DriveScanChange(Sender: TObject);
    procedure DriveScanTerminate(Sender: TObject);
    procedure Redraw(Delayed: Boolean = False);
    procedure DoDraw;
    procedure UpdateInterface;
    procedure UpdateSpeedChart;
  public
    procedure SaveConfig;
    procedure LoadConfig;
    procedure Detect;
    procedure ShowBuffer(Buffer: array of Byte);
  end;

var
  FormMain: TFormMain;


implementation

{$R *.lfm}

uses
  UCore, UFormProject, UFormOperation, UFormAbout, UFormSettings;

resourcestring
  SModifiedFlag = '(modified)';
  SNewProject = 'New project';
  SSelectDriveForScan = 'Select drive for scan';
  SOpenStoredProject = 'Open stored scan project';
  SSaveProject = 'Save scan project';
  SBytes = 'bytes';
  SProjectModified = 'Project modified';
  SProjectModifiedQuestion = 'Project was modified. Do you want to save it?';

{ TFormMain }

procedure TFormMain.FormShow(Sender: TObject);
begin
  Detect;
  LoadConfig;
  Core.LoadConfig;
  Core.PersistentForm1.Load(Self);
  if (LastOpenedList1.Items.Count > 0) and FileExists(LastOpenedList1.Items[0]) then
    ProjectOpen(LastOpenedList1.Items[0])
  else begin
    AFileNew.Execute;
    Redraw;
    UpdateInterface;
  end;
  PageControl1.TabIndex := 0;
end;

procedure TFormMain.Image1Resize(Sender: TObject);
begin
  Redraw;
end;

procedure TFormMain.ListView1Data(Sender: TObject; Item: TListItem);
begin
  with Core.Project do
  if (Item.Index >= 0) and (Item.Index < Scans.Count) then
  with TDriveScan(Scans[Scans.Count - 1 - Item.Index]) do begin
    Item.Caption := GetName;
    Item.Data := TDriveScan(Scans[Scans.Count - 1 - Item.Index]);
    Item.SubItems.Add(DateTimeToStr(TimeStart));
    Item.SubItems.Add(DateTimeToStr(TimeEnd));
    Item.SubItems.Add(IntToStr(SectorStart));
    Item.SubItems.Add(IntToStr(SectorEnd));
  end;
end;

procedure TFormMain.ListView1SelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  if Selected then begin
    Core.Project.CurrentScan := TDriveScan(Item.Data);
    Redraw;
    ReloadOperationList;
  end;
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  PrefixMultiplier := TPrefixMultiplier.Create;
end;

procedure TFormMain.AScanStopExecute(Sender: TObject);
begin
  if Assigned(Core.Project) and Assigned(Core.Project.CurrentScan) then begin
    Core.Project.CurrentScan.Stop;
    UpdateInterface;
  end;
end;

procedure TFormMain.AShowAboutExecute(Sender: TObject);
begin
  FormAbout := TFormAbout.Create(nil);
  try
    FormAbout.ApplicationInfo := Core.ApplicationInfo1;
    FormAbout.ShowModal;
  finally
    FreeAndNil(FormAbout);
  end;
end;

procedure TFormMain.AShowSettingsExecute(Sender: TObject);
begin
  FormSettings := TFormSettings.Create(nil);
  try
    FormSettings.Load;
    if FormSettings.ShowModal = mrOK then
      FormSettings.Save;
  finally
    FreeAndNil(FormSettings);
  end;
end;

procedure TFormMain.AViewToolbarExecute(Sender: TObject);
begin
  ShowToolBar := not ShowToolBar;
  ToolBar1.Visible := ShowToolBar;
  AViewToolbar.Checked := ShowToolBar;
end;

procedure TFormMain.ComboBoxDriveChange(Sender: TObject);
begin
  with Core.Project do
  if (ComboBoxDrive.ItemIndex <> -1) and (DriveInfo.Path <> TDriveInfo(ComboBoxDrive.Items.Objects[ComboBoxDrive.ItemIndex]).Path) then begin
    with TDriveInfo(ComboBoxDrive.Items.Objects[ComboBoxDrive.ItemIndex]) do
    if Kind = dkFile then begin
      OpenDialog1.FileName := Path;
      if OpenDialog1.Execute and FileExists(OpenDialog1.FileName) then begin
        Path := OpenDialog1.FileName;
        Size := FileSize(Path);
        Model := OpenDialog1.FileName;
        SectorSize := 4096;
        Core.DriveList.LoadToStrings(ComboBoxDrive.Items);
      end;
    end;
    DriveInfo.Assign(TDriveInfo(ComboBoxDrive.Items.Objects[ComboBoxDrive.ItemIndex]));
    Modified := True;
    Core.Project.ScanProfile.LoadFromDriveInfo(DriveInfo);
    UpdateInterface;
    Redraw;
  end;
end;

procedure TFormMain.AScanStartExecute(Sender: TObject);
begin
  LastBlockPos := 0;
  with Core.Project do begin
    Core.Project.Scans.Add(TDriveScan.Create);
    Core.Project.CurrentScan := TDriveScan(Core.Project.Scans.Last);
    CurrentScan.DriveName := DriveInfo.Path;
    CurrentScan.SectorCount := DriveInfo.SectorCount;
    CurrentScan.SectorSize := DriveInfo.SectorSize;
    Core.Project.ScanProfile.SectorCount := Core.Project.DriveInfo.SectorCount;
    CurrentScan.LoadProfile(Core.Project.ScanProfile);;
    CurrentScan.Reset;
    CurrentScan.OnChange := DriveScanChange;
    CurrentScan.OnTerminate := DriveScanTerminate;
    CurrentScan.Start;
    Modified := True;
  end;
  UpdateInterface;
end;

procedure TFormMain.AProjectOptionsExecute(Sender: TObject);
begin
  FormProject.Load(Core.Project);
  if FormProject.ShowModal = mrOk then begin
    FormProject.Save(Core.Project);
    Core.Project.Modified := True;
    UpdateInterface;
  end;
end;

procedure TFormMain.AScanContinueExecute(Sender: TObject);
begin
  Core.Project.CurrentScan.Start;
  Core.Project.Modified := True;
  UpdateInterface;
end;

procedure TFormMain.AExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.AFileCloseExecute(Sender: TObject);
begin
  FreeAndNil(Core.Project);
  UpdateInterface;
end;

procedure TFormMain.AFileNewExecute(Sender: TObject);
begin
  AFileClose.Execute;
  Core.Project := TProject.Create;
  Core.Project.FileName := SNewProject;
  Core.Project.Modified := False;

  ComboBoxDrive.ItemIndex := Core.DriveList.IndexOf(Core.DriveList.FindByModel(Core.Project.DriveInfo.Model));
  if (ComboBoxDrive.ItemIndex = -1) and (ComboBoxDrive.Items.Count > 0) then ComboBoxDrive.ItemIndex := 0;
  if ComboBoxDrive.ItemIndex <> -1 then
    Core.Project.DriveInfo.Assign(TDriveInfo(ComboBoxDrive.Items.Objects[ComboBoxDrive.ItemIndex]));
  Core.Project.ScanProfile.LoadFromDriveInfo(Core.Project.DriveInfo);

  Redraw;
  UpdateInterface;
end;

procedure TFormMain.AFileOpenExecute(Sender: TObject);
begin
  OpenDialog1.Title := SOpenStoredProject;
  OpenDialog1.DefaultExt := '.cdp';
  if Assigned(Core.Project) then
    OpenDialog1.FileName := Core.Project.FileName;
  if OpenDialog1.Execute then begin
    ProjectOpen(OpenDialog1.FileName);
  end;
end;

procedure TFormMain.AFileSaveAsExecute(Sender: TObject);
begin
  SaveDialog1.Title := SSaveProject;
  SaveDialog1.DefaultExt := '.cdp';
  SaveDialog1.FileName := Core.Project.FileName;
  if SaveDialog1.Execute then begin
    Core.Project.SaveToFile(SaveDialog1.FileName);
    LastOpenedList1.AddItem(SaveDialog1.FileName);
    LastOpenedList1.LoadToMenuItem(MenuItemOpenRecent, OpenRecentClick);
    UpdateInterface;
  end;
end;

procedure TFormMain.AFileSaveExecute(Sender: TObject);
begin
  if not FileExists(Core.Project.FileName) then
    AFileSaveAs.Execute
    else begin
      Core.Project.SaveToFile(Core.Project.FileName);
      LastOpenedList1.AddItem(SaveDialog1.FileName);
      LastOpenedList1.LoadToMenuItem(MenuItemOpenRecent, OpenRecentClick);
      UpdateInterface;
    end;
end;

procedure TFormMain.AOperationAddExecute(Sender: TObject);
var
  NewOperation: TDriveScan;
begin
  with Core.Project do begin
    NewOperation := TDriveScan.Create;
    Scans.Add(NewOperation);
    CurrentScan := TDriveScan(Scans.Last);
    CurrentScan.LoadProfile(ScanProfile);
    CurrentScan.Mode := rmNone;
    ReloadOperationList;
    Redraw;
    Core.Project.Modified := True;
  end;
end;

procedure TFormMain.AOperationOptionsExecute(Sender: TObject);
begin
  Core.Project.ScanProfile.SectorCount := Core.Project.DriveInfo.SectorCount;
  FormOperation.Load(Core.Project.ScanProfile);
  if FormOperation.ShowModal = mrOk then begin
    FormOperation.Save(Core.Project.ScanProfile);
    Core.Project.Modified := True;
    UpdateInterface;
  end;
end;

procedure TFormMain.AOperationRemoveExecute(Sender: TObject);
begin
  Core.Project.CurrentScan := nil;
  Core.Project.Scans.Remove(ListView1.Selected.Data);
  ReloadOperationList;
  Redraw;
  Core.Project.Modified := True;
end;

procedure TFormMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  AFileClose.Execute;
  SaveConfig;
  Core.SaveConfig;
  Core.PersistentForm1.Save(Self);
end;

procedure TFormMain.FormCloseQuery(Sender: TObject; var CanClose: boolean);
var
  R: TModalResult;
begin
  if Assigned(Core.Project) and Core.Project.Modified then begin
    R := MessageDlg(SProjectModified, SProjectModifiedQuestion, mtConfirmation, [mbYes, mbNo, mbCancel], 0);
    if R = mrCancel then begin
      CanClose := False;
      Exit;
    end else
    if R = mrYes then begin
      AFileSave.Execute;
    end;
  end;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  Timer1.Enabled := False;
  FreeAndNil(PrefixMultiplier);
end;

procedure TFormMain.Redraw(Delayed: Boolean = False);
begin
  RedrawPending := True;
  if not Delayed then DoDraw;
end;

procedure TFormMain.DoDraw;
begin
  if Assigned(Core.Project) and Assigned(Core.Project.CurrentScan) then
  with Core.Project do begin
    if RedrawPending then
    with Image1 do begin
      if (Width <> Picture.Bitmap.Width) or (Height <> Picture.Bitmap.Height) then
        Picture.Bitmap.SetSize(Width, Height);
      try
        Core.Project.CurrentScan.Lock.Acquire;
        try
          Picture.Bitmap.BeginUpdate(True);
          CurrentScan.BlockMap.Draw(Picture.Bitmap.Canvas);
        finally
          Picture.Bitmap.EndUpdate;
        end;
        LabelSectorPerBlock.Caption := IntToStr(CurrentScan.BlockMap.SectorPerBlock);
        LabelBlockCurrent.Caption := IntToStr(CurrentScan.SectorCurrent);
        LabelBlockDamaged.Caption := IntToStr(CurrentScan.DamagedBlockCount);
      finally
        CurrentScan.Lock.Release;
      end;
      RedrawPending := False;
    end;
    try
      CurrentScan.Lock.Acquire;
      if not CurrentScan.Terminated then begin
        LabelElapsedTime.Caption := TimeToStr(CurrentScan.GetElapsedTime);
        LabelEstimatedTime.Caption := TimeToStr((Now - CurrentScan.TimeStart) /
          CurrentScan.SectorCurrent * (CurrentScan.BlockMap.SectorCount - CurrentScan.SectorCurrent));
        LabelIOSpeed.Caption := PrefixMultiplier.Add((CurrentScan.SectorCurrent - LastBlockPos) *
          CurrentScan.SectorSize / (Timer1.Interval / 1000), BasePrefixMultipliers, 'B/s');
        LastBlockPos := CurrentScan.SectorCurrent;
      end else begin
        LabelElapsedTime.Caption := '';
        LabelEstimatedTime.Caption := '';
        LabelIOSpeed.Caption := '';
      end;
    finally
      CurrentScan.Lock.Release;
    end;
    UpdateSpeedChart;
  end else
  with Image1 do begin
    if (Width <> Picture.Bitmap.Width) or (Height <> Picture.Bitmap.Height) then
      Picture.Bitmap.SetSize(Width, Height);
    with Picture.Bitmap.Canvas do begin
      Brush.Style := bsSolid;
      Brush.Color := clSilver;
      FillRect(0, 0, Width, Height);
    end;
  end;
end;

procedure TFormMain.UpdateInterface;
var
  Title: string;
begin
  with Core do begin
  if Assigned(Project) then begin
    AScanStart.Enabled := (Assigned(Project.CurrentScan) and (Project.CurrentScan.Terminated = True)) or not Assigned(Project.CurrentScan);
    AScanStop.Enabled := Assigned(Project.CurrentScan) and (Project.CurrentScan.Terminated = False);
    AScanContinue.Enabled := Assigned(Project.CurrentScan) and (Project.CurrentScan.Terminated = True) and
      (Project.CurrentScan.SectorCurrent > Project.CurrentScan.SectorStart) and
      (Project.CurrentScan.SectorCurrent < Project.CurrentScan.SectorEnd);
  end else begin
    AScanStart.Enabled := False;
    AScanStop.Enabled := False;
    AScanContinue.Enabled := False;
  end;
  AFileSave.Enabled := Assigned(Project) and Project.Modified;
  AFileSaveAs.Enabled := Assigned(Project);
  AFileClose.Enabled := Assigned(Project);
  ComboBoxDrive.Enabled := Assigned(Project);
  AProjectOptions.Enabled := Assigned(Project);;
  AOperationOptions.Enabled := Assigned(Project);;
  ReloadOperationList;

  Title := '';
  if Assigned(Project) then
    Title := Title + ExtractFileName(Project.FileName) + ' - ';
  Title := Title + Application.Title;
  if Assigned(Project) and Project.Modified then
    Title := Title + ' ' + SModifiedFlag;
  Caption := Title;

  if Assigned(Project) then begin
    LabelSize.Caption := PrefixMultiplier.Add(Project.DriveInfo.Size,
      BasePrefixMultipliers, SBytes);
    LabelSectorCount.Caption := IntToStr(Project.DriveInfo.SectorCount);
    LabelSectorSize.Caption := PrefixMultiplier.Add(Project.DriveInfo.SectorSize, BasePrefixMultipliers, SBytes);
  end else begin
    LabelSize.Caption := '';
    LabelSectorCount.Caption := '';
    LabelSectorSize.Caption := '';
  end;
  end;
end;

procedure TFormMain.UpdateSpeedChart;
var
  I: Integer;
begin
  if TabSheetSpeed.Visible then
  if Assigned(Core.Project) then
  with Core.Project do
  if Assigned(CurrentScan) then
  with CurrentScan do begin
    ChartSpeed.BottomAxis.Range.UseMax := True;
    ChartSpeed.BottomAxis.Range.Max := Length(SpeedSteps);
    ChartSpeedLineSeriesAvg.Clear;
    ChartSpeedLineSeriesMin.Clear;
    ChartSpeedLineSeriesMax.Clear;
    for I := 0 to Length(SpeedSteps) - 1 do
    if not SpeedSteps[I].Null then begin
      ChartSpeedLineSeriesAvg.AddXY(I, SpeedSteps[I].Average / (1024 * 1024));
      ChartSpeedLineSeriesMin.AddXY(I, SpeedSteps[I].Min / (1024 * 1024));
      ChartSpeedLineSeriesMax.AddXY(I, SpeedSteps[I].Max / (1024 * 1024));
    end;
  end;
end;

procedure TFormMain.SaveConfig;
begin
  Core.XMLConfig1.SetValue('DriveName', LastDriveName);
  LastOpenedList1.SaveToXMLConfig(Core.XMLConfig1, 'RecentProjects');
  Core.XMLConfig1.SetValue('ShowToolBar', ShowToolBar);
end;

procedure TFormMain.LoadConfig;
begin
  LastDriveName := Core.XMLConfig1.GetValue('DriveName', '');
  LastOpenedList1.LoadFromXMLConfig(Core.XMLConfig1, 'RecentProjects');
  ShowToolBar := Core.XMLConfig1.GetValue('ShowToolBar', False);
  ToolBar1.Visible := ShowToolBar;
  AViewToolbar.Checked := ShowToolBar;
end;

procedure TFormMain.Timer1Timer(Sender: TObject);
begin
  Redraw;
end;

procedure TFormMain.OpenRecentClick(Sender: TObject);
begin
  ProjectOpen(TMenuItem(Sender).Caption);
end;

procedure TFormMain.ReloadOperationList;
begin
  if Assigned(Core.Project) then begin
    ListView1.Items.Count := Core.Project.Scans.Count;
  end else begin
    ListView1.Items.Count := 0;
  end;
  ListView1.Refresh;

  AOperationRemove.Enabled := Assigned(ListView1.Selected);
end;

procedure TFormMain.ProjectOpen(FileName: string);
begin
  AFileNew.Execute;
  Core.Project.LoadFromFile(FileName);
  ComboBoxDrive.ItemIndex := Core.DriveList.IndexOf(Core.DriveList.FindByModel(Core.Project.DriveInfo.Model));
  if (ComboBoxDrive.ItemIndex = -1) and (ComboBoxDrive.Items.Count > 0) then ComboBoxDrive.ItemIndex := 0;
  LastOpenedList1.AddItem(FileName);
  LastOpenedList1.LoadToMenuItem(MenuItemOpenRecent, OpenRecentClick);
  UpdateInterface;
  Redraw;
end;

procedure TFormMain.DriveScanChange(Sender: TObject);
begin
  RedrawPending := True;
end;

procedure TFormMain.DriveScanTerminate(Sender: TObject);
begin
  UpdateInterface;
end;

procedure TFormMain.Detect;
begin
  Core.DriveList.Detect;
  Core.DriveList.LoadToStrings(ComboBoxDrive.Items);
end;

procedure TFormMain.ShowBuffer(Buffer: array of Byte);
const
  BytePerRow = 16;
var
  Row: string;
  I: Integer;
  Item: TListItem;
begin
  ListView1.Clear;
  Row := '';
  for I := 0 to Length(Buffer) do begin
    Row := Row + IntToHex(Buffer[I], 2) + ' ';
    if ((I mod BytePerRow) = (BytePerRow - 1)) or (I = (Length(Buffer) - 1)) then begin
      Item := ListView1.Items.Add;
      Item.Caption := '';
      Item.SubItems.Add(Row);
      Row := '';
    end;
  end;
end;

end.

