unit UFormOperation;

{$mode delphi}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Spin, UDriveScan;

type

  { TFormOperation }

  TFormOperation = class(TForm)
    ButtonOk: TButton;
    ButtonCancel: TButton;
    CheckBoxRandomPattern: TCheckBox;
    ComboBoxRunMode: TComboBox;
    EditPattern: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    Label9: TLabel;
    SpinEditFirstSector: TSpinEdit;
    SpinEditLastSector: TSpinEdit;
    procedure CheckBoxRandomPatternChange(Sender: TObject);
    procedure ComboBoxRunModeChange(Sender: TObject);
    procedure SpinEditFirstSectorChange(Sender: TObject);
    procedure SpinEditLastSectorChange(Sender: TObject);
  private
    procedure UpdateMaxValues;
    procedure UpdateWritePattern;
  public
    procedure Load(DriveScan: TDriveScanProfile);
    procedure Save(DriveScan: TDriveScanProfile);
  end;

var
  FormOperation: TFormOperation;

implementation

{$R *.lfm}

{ TFormOperation }

procedure TFormOperation.SpinEditFirstSectorChange(Sender: TObject);
begin
  UpdateMaxValues;
end;

procedure TFormOperation.ComboBoxRunModeChange(Sender: TObject);
begin
  UpdateWritePattern;
end;

procedure TFormOperation.CheckBoxRandomPatternChange(Sender: TObject);
begin
  UpdateWritePattern;
end;

procedure TFormOperation.SpinEditLastSectorChange(Sender: TObject);
begin
  UpdateMaxValues;
end;

procedure TFormOperation.UpdateMaxValues;
begin
  SpinEditFirstSector.MaxValue := SpinEditLastSector.Value;
  SpinEditLastSector.MinValue := SpinEditFirstSector.Value;
end;

procedure TFormOperation.UpdateWritePattern;
begin
  CheckBoxRandomPattern.Enabled := (ComboBoxRunMode.ItemIndex = Integer(rmWrite));
  EditPattern.Enabled := not CheckBoxRandomPattern.Checked and (ComboBoxRunMode.ItemIndex = Integer(rmWrite));
end;

procedure TFormOperation.Load(DriveScan: TDriveScanProfile);
begin
  CheckBoxRandomPattern.Checked := DriveScan.WritePatternRandom;
  UpdateWritePattern;
  ComboBoxRunMode.ItemIndex := Integer(DriveScan.Mode);
  EditPattern.Text := '$' + IntToHex(DriveScan.WritePattern, 2);
  SpinEditLastSector.MaxValue := DriveScan.SectorCount - 1;
  SpinEditFirstSector.Value := DriveScan.SectorStart;
  SpinEditLastSector.Value := DriveScan.SectorEnd;
  UpdateMaxValues;
end;

procedure TFormOperation.Save(DriveScan: TDriveScanProfile);
begin
  DriveScan.Mode := TRunMode(ComboBoxRunMode.ItemIndex);
  DriveScan.WritePattern := StrToInt(EditPattern.Text);
  DriveScan.SectorStart := SpinEditFirstSector.Value;
  DriveScan.SectorEnd := SpinEditLastSector.Value;
  DriveScan.WritePatternRandom := CheckBoxRandomPattern.Checked;
end;

end.

