unit UCore;

{$mode delphi}

interface

uses
  Classes, SysUtils, XMLConf, FileUtil, Controls, UProject, UPhysDrive,
  UCoolTranslator, UPersistentForm, UApplicationInfo, URegistry, UScaleDPI,
  Registry;

type

  { TCore }

  TCore = class(TDataModule)
    ApplicationInfo1: TApplicationInfo;
    CoolTranslator1: TCoolTranslator;
    ImageList1: TImageList;
    PersistentForm1: TPersistentForm;
    ScaleDPI1: TScaleDPI;
    XMLConfig1: TXMLConfig;
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    RegistryContext: TRegistryContext;
  public
    DevelMode: Boolean;
    DriveList: TDriveList;
    Project: TProject;
    procedure LoadConfig;
    procedure SaveConfig;
  end;

var
  Core: TCore;

implementation

{$R *.lfm}

{ TCore }

procedure TCore.DataModuleCreate(Sender: TObject);
begin
  DriveList := TDriveList.Create;
  Project := nil;
  XMLConfig1.Filename := 'config.xml';
  Randomize;
end;

procedure TCore.DataModuleDestroy(Sender: TObject);
begin
  if Assigned(Project) then FreeAndNil(Project);
  FreeAndNil(DriveList);
end;

procedure TCore.LoadConfig;
begin
  RegistryContext := RegContext(HKEY(ApplicationInfo1.RegistryRoot), ApplicationInfo1.RegistryKey);
  PersistentForm1.RegistryContext := RegistryContext;
  XMLConfig1.Filename := GetAppConfigDir(False) + 'Config.xml';
  ForceDirectories(ExtractFileDir(XMLConfig1.Filename));

  CoolTranslator1.Language := CoolTranslator1.Languages.SearchByCode(XMLConfig1.GetValue('Language', ''));
end;

procedure TCore.SaveConfig;
begin
  XMLConfig1.SetValue('Language', CoolTranslator1.Language.Code);
end;

end.

